@@ -1,11 +1,14 @@
 # SPDX-FileCopyrightText: 2019 Free Software Foundation Europe e.V. <https://fsfe.org>
 # SPDX-FileCopyrightText: 2019 Kirill Elagin
 # SPDX-FileCopyrightText: 2020 Dmitry Bogatov
-# SPDX-FileCopyrightText: 2021 Alliander N.V. <https://alliander.com>
+# SPDX-FileCopyrightText: 2021-2022 Alliander N.V. <https://alliander.com>
 # SPDX-FileCopyrightText: 2021 Alvar Penning
 # SPDX-FileCopyrightText: 2021 Robin Vobruba <hoijui.quaero@gmail.com>
 # SPDX-FileCopyrightText: 2021 Matija Šuklje <matija@suklje.name>
 # SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
+# SPDX-FileCopyrightText: 2022 Nico Rikken <nico.rikken@fsfe.org>
+# SPDX-FileCopyrightText: 2022 Stefan Hynek <stefan.hynek@uni-goettingen.de>
+# SPDX-FileCopyrightText: 2022 Carmen Bianca Bakker <carmenbianca@fsfe.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -16,7 +19,7 @@ headers, in any case.
 import logging
 import operator
 from textwrap import dedent
-from typing import List
+from typing import List, NamedTuple
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -29,6 +32,16 @@ class CommentCreateError(Exception):
     """An error occurred during the creation of a comment."""
 
 
+class MultiLineSegments(NamedTuple):
+    """Components that make up a multi-line comment style, e.g. '/*', '*', and
+    '*/'.
+    """
+
+    start: str
+    middle: str
+    end: str
+
+
 class CommentStyle:
     """Base class for comment style."""
 
@@ -36,10 +49,11 @@ class CommentStyle:
     INDENT_AFTER_SINGLE = ""
     # (start, middle, end)
     # e.g., ("/*", "*", "*/")
-    MULTI_LINE = ("", "", "")
+    MULTI_LINE = MultiLineSegments("", "", "")
     INDENT_BEFORE_MIDDLE = ""
     INDENT_AFTER_MIDDLE = ""
     INDENT_BEFORE_END = ""
+    SHEBANGS = []
 
     @classmethod
     def can_handle_single(cls) -> bool:
@@ -49,7 +63,7 @@ class CommentStyle:
     @classmethod
     def can_handle_multi(cls) -> bool:
         """Whether the :class:`CommentStyle` can handle multi-line comments."""
-        return all((cls.MULTI_LINE[0], cls.MULTI_LINE[2]))
+        return all((cls.MULTI_LINE.start, cls.MULTI_LINE.end))
 
     @classmethod
     def create_comment(cls, text: str, force_multi: bool = False) -> str:
@@ -89,19 +103,19 @@ class CommentStyle:
         if not cls.can_handle_multi():
             raise CommentCreateError(f"{cls} cannot create multi-line comments")
         result = []
-        result.append(cls.MULTI_LINE[0])
+        result.append(cls.MULTI_LINE.start)
         for line in text.split("\n"):
-            if cls.MULTI_LINE[2] in text:
+            if cls.MULTI_LINE.end in text:
                 raise CommentCreateError(
                     f"'{line}' contains a premature comment delimiter"
                 )
             line_result = ""
-            if cls.MULTI_LINE[1]:
-                line_result += cls.INDENT_BEFORE_MIDDLE + cls.MULTI_LINE[1]
+            if cls.MULTI_LINE.middle:
+                line_result += cls.INDENT_BEFORE_MIDDLE + cls.MULTI_LINE.middle
             if line:
                 line_result += cls.INDENT_AFTER_MIDDLE + line
             result.append(line_result)
-        result.append(cls.INDENT_BEFORE_END + cls.MULTI_LINE[2])
+        result.append(cls.INDENT_BEFORE_END + cls.MULTI_LINE.end)
         return "\n".join(result)
 
     @classmethod
@@ -138,9 +152,9 @@ class CommentStyle:
 
     @classmethod
     def _remove_middle_marker(cls, line: str) -> str:
-        if cls.MULTI_LINE[1]:
+        if cls.MULTI_LINE.middle:
             possible_line = line.lstrip()
-            prefix = cls.MULTI_LINE[1]
+            prefix = cls.MULTI_LINE.middle
             if possible_line.startswith(prefix):
                 line = possible_line.lstrip(prefix)
                 # Note to future self: line.removeprefix would be preferable
@@ -173,11 +187,11 @@ class CommentStyle:
             last = None  # Set this later.
             last_is_first = True
 
-        if not first.startswith(cls.MULTI_LINE[0]):
+        if not first.startswith(cls.MULTI_LINE.start):
             raise CommentParseError(
                 f"'{first}' does not start with a comment marker"
             )
-        first = first.lstrip(cls.MULTI_LINE[0])
+        first = first.lstrip(cls.MULTI_LINE.start)
         first = first.lstrip()
 
         for line in lines:
@@ -187,11 +201,11 @@ class CommentStyle:
         if last_is_first:
             last = first
             first = ""
-        if not last.endswith(cls.MULTI_LINE[2]):
+        if not last.endswith(cls.MULTI_LINE.end):
             raise CommentParseError(
                 f"'{last}' does not end with a comment delimiter"
             )
-        last = last.rstrip(cls.MULTI_LINE[2])
+        last = last.rstrip(cls.MULTI_LINE.end)
         last = last.rstrip()
         last = cls._remove_middle_marker(last)
 
@@ -222,11 +236,11 @@ class CommentStyle:
                     break
                 end = i
             return "\n".join(lines[0 : end + 1])
-        if cls.can_handle_multi() and text.startswith(cls.MULTI_LINE[0]):
+        if cls.can_handle_multi() and text.startswith(cls.MULTI_LINE.start):
             end = 0
             for i, line in enumerate(lines):
                 end = i
-                if line.endswith(cls.MULTI_LINE[2]):
+                if line.endswith(cls.MULTI_LINE.end):
                     break
             else:
                 raise CommentParseError("Comment block never delimits")
@@ -244,7 +258,7 @@ class AppleScriptCommentStyle(CommentStyle):
 
     SINGLE_LINE = "--"
     INDENT_AFTER_SINGLE = " "
-    MULTI_LINE = ("(*", "", "*)")
+    MULTI_LINE = MultiLineSegments("(*", "", "*)")
 
 
 class AspxCommentStyle(CommentStyle):
@@ -252,7 +266,7 @@ class AspxCommentStyle(CommentStyle):
 
     _shorthand = "aspx"
 
-    MULTI_LINE = ("<%--", "", "--%>")
+    MULTI_LINE = MultiLineSegments("<%--", "", "--%>")
 
 
 class BatchFileCommentStyle(CommentStyle):
@@ -269,7 +283,7 @@ class BibTexCommentStyle(CommentStyle):
 
     _shorthand = "bibtex"
 
-    MULTI_LINE = ("@Comment{", "", "}")
+    MULTI_LINE = MultiLineSegments("@Comment{", "", "}")
 
 
 class CCommentStyle(CommentStyle):
@@ -279,10 +293,14 @@ class CCommentStyle(CommentStyle):
 
     SINGLE_LINE = "//"
     INDENT_AFTER_SINGLE = " "
-    MULTI_LINE = ("/*", "*", "*/")
+    MULTI_LINE = MultiLineSegments("/*", "*", "*/")
     INDENT_BEFORE_MIDDLE = " "
     INDENT_AFTER_MIDDLE = " "
     INDENT_BEFORE_END = " "
+    SHEBANGS = [
+        "#!",  #  V-Lang
+        "<?php",  # PHP
+    ]
 
 
 class CssCommentStyle(CommentStyle):
@@ -290,7 +308,7 @@ class CssCommentStyle(CommentStyle):
 
     _shorthand = "css"
 
-    MULTI_LINE = ("/*", "*", "*/")
+    MULTI_LINE = MultiLineSegments("/*", "*", "*/")
     INDENT_BEFORE_MIDDLE = " "
     INDENT_AFTER_MIDDLE = " "
     INDENT_BEFORE_END = " "
@@ -326,7 +344,7 @@ class FtlCommentStyle(CommentStyle):
 
     _shorthand = "ftl"
 
-    MULTI_LINE = ("<#--", "", "-->")
+    MULTI_LINE = MultiLineSegments("<#--", "", "-->")
 
 
 class HandlebarsCommentStyle(CommentStyle):
@@ -334,7 +352,7 @@ class HandlebarsCommentStyle(CommentStyle):
 
     _shorthand = "handlebars"
 
-    MULTI_LINE = ("{{!--", "", "--}}")
+    MULTI_LINE = MultiLineSegments("{{!--", "", "--}}")
 
 
 class HaskellCommentStyle(CommentStyle):
@@ -351,7 +369,8 @@ class HtmlCommentStyle(CommentStyle):
 
     _shorthand = "html"
 
-    MULTI_LINE = ("<!--", "", "-->")
+    MULTI_LINE = MultiLineSegments("<!--", "", "-->")
+    SHEBANGS = ["<?xml"]
 
 
 class JinjaCommentStyle(CommentStyle):
@@ -359,7 +378,7 @@ class JinjaCommentStyle(CommentStyle):
 
     _shorthand = "jinja"
 
-    MULTI_LINE = ("{#", "", "#}")
+    MULTI_LINE = MultiLineSegments("{#", "", "#}")
 
 
 class LispCommentStyle(CommentStyle):
@@ -385,7 +404,7 @@ class MlCommentStyle(CommentStyle):
 
     _shorthand = "ml"
 
-    MULTI_LINE = ("(*", "*", "*)")
+    MULTI_LINE = MultiLineSegments("(*", "*", "*)")
     INDENT_BEFORE_MIDDLE = " "
     INDENT_AFTER_MIDDLE = " "
     INDENT_BEFORE_END = " "
@@ -398,7 +417,7 @@ class PlantUmlCommentStyle(CommentStyle):
 
     SINGLE_LINE = "'"
     INDENT_AFTER_SINGLE = " "
-    MULTI_LINE = ("/'", "'", "'/")
+    MULTI_LINE = MultiLineSegments("/'", "'", "'/")
     INDENT_BEFORE_MIDDLE = " "
     INDENT_AFTER_MIDDLE = " "
     INDENT_BEFORE_END = " "
@@ -411,6 +430,7 @@ class PythonCommentStyle(CommentStyle):
 
     SINGLE_LINE = "#"
     INDENT_AFTER_SINGLE = " "
+    SHEBANGS = ["#!"]
 
 
 class ReStructedTextCommentStyle(CommentStyle):
@@ -437,6 +457,15 @@ class UncommentableCommentStyle(EmptyCommentStyle):
     """
 
 
+class VelocityCommentStyle(CommentStyle):
+    """Apache Velocity Template Language comment style."""
+
+    _shorthand = "vst"
+
+    # TODO: SINGLE_LINE requires refactor to support trailing `**`.
+    MULTI_LINE = MultiLineSegments("#*", "  ", "*#")
+
+
 class VimCommentStyle(CommentStyle):
     """Vim(Script|Config) style."""
 
@@ -446,6 +475,17 @@ class VimCommentStyle(CommentStyle):
     INDENT_AFTER_SINGLE = " "
 
 
+class XQueryCommentStyle(CommentStyle):
+    """XQuery comment style."""
+
+    _shorthand = "xquery"
+
+    MULTI_LINE = MultiLineSegments("(:", ":", ":)")
+    INDENT_BEFORE_MIDDLE = " "
+    INDENT_AFTER_MIDDLE = " "
+    INDENT_BEFORE_END = " "
+
+
 #: A map of (common) file extensions against comment types.
 EXTENSION_COMMENT_STYLE_MAP = {
     ".adb": HaskellCommentStyle,
@@ -453,6 +493,7 @@ EXTENSION_COMMENT_STYLE_MAP = {
     ".ads": HaskellCommentStyle,
     ".ahk": LispCommentStyle,
     ".ahkl": LispCommentStyle,
+    ".aidl": CCommentStyle,
     ".applescript": AppleScriptCommentStyle,
     ".asax": AspxCommentStyle,
     ".asc": CCommentStyle,
@@ -511,8 +552,9 @@ EXTENSION_COMMENT_STYLE_MAP = {
     ".graphql": PythonCommentStyle,
     ".groovy": CCommentStyle,
     ".h": CCommentStyle,
-    ".hh": CCommentStyle,
+    ".ha": CCommentStyle,
     ".hbs": HandlebarsCommentStyle,
+    ".hh": CCommentStyle,
     ".hpp": CCommentStyle,
     ".hrl": TexCommentStyle,
     ".hs": HaskellCommentStyle,
@@ -532,8 +574,10 @@ EXTENSION_COMMENT_STYLE_MAP = {
     ".jy": PythonCommentStyle,
     ".ksh": PythonCommentStyle,
     ".kt": CCommentStyle,
+    ".kts": CCommentStyle,
     ".l": LispCommentStyle,
     ".latex": TexCommentStyle,
+    ".less": CssCommentStyle,
     ".license": EmptyCommentStyle,
     ".lisp": LispCommentStyle,
     ".lsp": LispCommentStyle,
@@ -562,6 +606,7 @@ EXTENSION_COMMENT_STYLE_MAP = {
     ".ots": UncommentableCommentStyle,
     ".ott": UncommentableCommentStyle,
     ".pdf": UncommentableCommentStyle,
+    ".pem": UncommentableCommentStyle,
     ".php": CCommentStyle,
     ".php3": CCommentStyle,
     ".php4": CCommentStyle,
@@ -623,11 +668,18 @@ EXTENSION_COMMENT_STYLE_MAP = {
     ".v": CCommentStyle,  # V-Lang source code
     ".vala": CCommentStyle,
     ".vim": VimCommentStyle,
+    ".vm": VelocityCommentStyle,
     ".vsh": CCommentStyle,  # V-Lang script
+    ".vtl": VelocityCommentStyle,
     ".vue": HtmlCommentStyle,
     ".xls": UncommentableCommentStyle,
     ".xlsx": UncommentableCommentStyle,
     ".xml": HtmlCommentStyle,
+    ".xq": XQueryCommentStyle,
+    ".xql": XQueryCommentStyle,
+    ".xqm": XQueryCommentStyle,
+    ".xqy": XQueryCommentStyle,
+    ".xquery": XQueryCommentStyle,
     ".xsd": HtmlCommentStyle,
     ".xsh": PythonCommentStyle,
     ".xsl": HtmlCommentStyle,
@@ -658,8 +710,10 @@ FILENAME_COMMENT_STYLE_MAP = {
     ".Rprofile": PythonCommentStyle,
     ".vimrc": VimCommentStyle,
     ".yarnrc": PythonCommentStyle,
+    "ansible.cfg": PythonCommentStyle,
     "archive.sctxar": UncommentableCommentStyle,  # SuperCollider global archive
     "CMakeLists.txt": PythonCommentStyle,
+    "CODEOWNERS": PythonCommentStyle,
     "configure.ac": M4CommentStyle,
     "Containerfile": PythonCommentStyle,
     "Dockerfile": PythonCommentStyle,
