@@ -1,4 +1,5 @@
 # SPDX-FileCopyrightText: 2021 Free Software Foundation Europe e.V. <https://fsfe.org>
+# SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,13 +10,13 @@ import sys
 from ._licenses import _LICENSES, _load_license_list
 from .project import Project
 
+
 # pylint: disable=unused-argument
-
-
 def add_arguments(parser) -> None:
     """Add arguments to the parser."""
 
 
+# pylint: disable=unused-argument
 def run(args, project: Project, out=sys.stdout):
     """Print the supported SPDX licenses list"""
 
@@ -25,9 +26,8 @@ def run(args, project: Project, out=sys.stdout):
         license_name = license_info["name"]
         license_reference = license_info["reference"]
         out.write(
-            "{: <40}\t{: <80}\t{: <50}\n".format(
-                license_id, license_name, license_reference
-            )
+            f"{license_id: <40}\t{license_name: <80}\t"
+            f"{license_reference: <50}\n"
         )
 
     return 0
