@@ -1,11 +1,12 @@
 # SPDX-FileCopyrightText: 2017 Free Software Foundation Europe e.V. <https://fsfe.org>
 # SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
+# SPDX-FileCopyrightText: 2022 Carmen Bianca Bakker <carmenbianca@fsfe.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 """Global fixtures and configuration."""
 
-# pylint: disable=redefined-outer-name,subprocess-run-check
+# pylint: disable=redefined-outer-name
 
 import datetime
 import logging
@@ -42,6 +43,8 @@ CWD = Path.cwd()
 TESTS_DIRECTORY = Path(__file__).parent.resolve()
 RESOURCES_DIRECTORY = TESTS_DIRECTORY / "resources"
 
+# REUSE-IgnoreStart
+
 
 def pytest_configure():
     """Called after command line options have been parsed and all plugins and
@@ -106,14 +109,12 @@ def fake_repository(tmpdir_factory) -> Path:
     # Adding this here to avoid conflict in main project.
     (directory / "src/exception.py").write_text(
         "SPDX-FileCopyrightText: 2017 Jane Doe\n"
-        "SPDX"
-        "-License-Identifier: GPL-3.0-or-later WITH Autoconf-exception-3.0",
+        "SPDX-License-Identifier: GPL-3.0-or-later WITH Autoconf-exception-3.0",
         encoding="utf-8",
     )
     (directory / "src/custom.py").write_text(
         "SPDX-FileCopyrightText: 2017 Jane Doe\n"
-        "SPDX"
-        "-License-Identifier: LicenseRef-custom",
+        "SPDX-License-Identifier: LicenseRef-custom",
         encoding="utf-8",
     )
 
@@ -132,10 +133,8 @@ def _repo_contents(
     with a prefix line in the ignore file.
     """
     gitignore = ignore_prefix + (
-        "# SPDX"
-        "-License-Identifier: CC0-1.0\n"
-        "# SPDX"
-        "-FileCopyrightText: 2017 Jane Doe\n"
+        "# SPDX-License-Identifier: CC0-1.0\n"
+        "# SPDX-FileCopyrightText: 2017 Jane Doe\n"
         "*.pyc\nbuild"
     )
     (fake_repository / ignore_filename).write_text(gitignore)
@@ -156,21 +155,21 @@ def git_repository(fake_repository: Path, git_exe: Optional[str]) -> Path:
     os.chdir(fake_repository)
     _repo_contents(fake_repository)
 
-    subprocess.run([git_exe, "init", str(fake_repository)])
-    subprocess.run([git_exe, "add", str(fake_repository)])
+    subprocess.run([git_exe, "init", str(fake_repository)], check=True)
+    subprocess.run([git_exe, "config", "user.name", "Example"], check=True)
+    subprocess.run(
+        [git_exe, "config", "user.email", "example@example.com"], check=True
+    )
+
+    subprocess.run([git_exe, "add", str(fake_repository)], check=True)
     subprocess.run(
         [
             git_exe,
-            "-c",
-            "user.name",
-            "Example",
-            "-c",
-            "user.email",
-            "example@example.com",
             "commit",
             "-m",
             "initial",
-        ]
+        ],
+        check=True,
     )
 
     return fake_repository
@@ -186,8 +185,8 @@ def hg_repository(fake_repository: Path, hg_exe: Optional[str]) -> Path:
         ignore_prefix="syntax:glob",
     )
 
-    subprocess.run([hg_exe, "init", "."])
-    subprocess.run([hg_exe, "addremove"])
+    subprocess.run([hg_exe, "init", "."], check=True)
+    subprocess.run([hg_exe, "addremove"], check=True)
     subprocess.run(
         [
             hg_exe,
@@ -196,7 +195,8 @@ def hg_repository(fake_repository: Path, hg_exe: Optional[str]) -> Path:
             "Example <example@example.com>",
             "-m",
             "initial",
-        ]
+        ],
+        check=True,
     )
 
     return fake_repository
@@ -209,50 +209,61 @@ def submodule_repository(
     """Create a git repository that contains a submodule."""
     header = cleandoc(
         """
-            spdx-FileCopyrightText: 2019 Jane Doe
+            SPDX-FileCopyrightText: 2019 Jane Doe
 
-            spdx-License-Identifier: CC0-1.0
+            SPDX-License-Identifier: CC0-1.0
             """
-    ).replace("spdx", "SPDX")
+    )
 
     submodule = Path(str(tmpdir_factory.mktemp("submodule")))
     (submodule / "foo.py").write_text(header, encoding="utf-8")
 
     os.chdir(submodule)
-    subprocess.run([git_exe, "init", str(submodule)])
-    subprocess.run([git_exe, "add", str(submodule)])
+
+    subprocess.run([git_exe, "init", str(submodule)], check=True)
+    subprocess.run([git_exe, "config", "user.name", "Example"], check=True)
+    subprocess.run(
+        [git_exe, "config", "user.email", "example@example.com"], check=True
+    )
+
+    subprocess.run([git_exe, "add", str(submodule)], check=True)
     subprocess.run(
         [
             git_exe,
-            "-c",
-            "user.name",
-            "Example",
-            "-c",
-            "user.email",
-            "example@example.com",
             "commit",
             "-m",
             "initial",
-        ]
+        ],
+        check=True,
     )
 
     os.chdir(git_repository)
+
     subprocess.run(
-        [git_exe, "submodule", "add", str(submodule.resolve()), "submodule"]
+        [
+            git_exe,
+            # https://git-scm.com/docs/git-config#Documentation/git-config.txt-protocolallow
+            #
+            # This circumvents a bug/behaviour caused by CVE-2022-39253 where
+            # you cannot use `git submodule add repository path` where
+            # repository is a file on the filesystem.
+            "-c",
+            "protocol.file.allow=always",
+            "submodule",
+            "add",
+            str(submodule.resolve()),
+            "submodule",
+        ],
+        check=True,
     )
     subprocess.run(
         [
             git_exe,
-            "-c",
-            "user.name",
-            "Example",
-            "-c",
-            "user.email",
-            "example@example.com",
             "commit",
             "-m",
             "add submodule",
-        ]
+        ],
+        check=True,
     )
 
     (git_repository / ".gitmodules.license").write_text(header)
@@ -293,7 +304,7 @@ def template_simple_source():
         {% endfor %}
 
         {% for expression in spdx_expressions %}
-        spdx-License-Identifier: {{ expression }}
+        SPDX-License-Identifier: {{ expression }}
         {% endfor %}
         """.replace(
             "spdx-Lic", "SPDX-Lic"
@@ -333,7 +344,7 @@ def template_commented_source():
         {% endfor %}
         #
         {% for expression in spdx_expressions %}
-        # spdx-License-Identifier: {{ expression }}
+        # SPDX-License-Identifier: {{ expression }}
         {% endfor %}
         """.replace(
             "spdx-Lic", "SPDX-Lic"
@@ -354,3 +365,6 @@ def mock_date_today(monkeypatch):
     date = create_autospec(datetime.date)
     date.today.return_value = datetime.date(2018, 1, 1)
     monkeypatch.setattr(datetime, "date", date)
+
+
+# REUSE-IgnoreEnd
