@@ -28,6 +28,9 @@ TESTS_DIRECTORY = Path(__file__).parent.resolve()
 RESOURCES_DIRECTORY = TESTS_DIRECTORY / "resources"
 
 
+# REUSE-IgnoreStart
+
+
 def test_project_not_a_directory(empty_directory):
     """Cannot create a Project without a valid directory."""
     (empty_directory / "foo.py").write_text("foo")
@@ -100,11 +103,11 @@ def test_all_files_symlinks(empty_directory):
     (empty_directory / "blob.license").write_text(
         cleandoc(
             """
-            spdx-FileCopyrightText: Jane Doe
+            SPDX-FileCopyrightText: Jane Doe
 
-            spdx-License-Identifier: GPL-3.0-or-later
+            SPDX-License-Identifier: GPL-3.0-or-later
             """
-        ).replace("spdx", "SPDX")
+        )
     )
     (empty_directory / "symlink").symlink_to("blob")
     project = Project(empty_directory)
@@ -375,3 +378,6 @@ def test_relative_from_root_no_shared_base_path(empty_directory):
     assert project.relative_from_root(
         Path(f"{project.root.name}/src/hello.py")
     ) == Path("src/hello.py")
+
+
+# REUSE-IgnoreEnd
